/** Rill v0.0.647 https://www.npmjs.com/package/rill License: MIT*/
"use strict";
var browser, byteLength, htmlReg, isType, noop, statuses;

statuses = require("statuses");

isType = require("@rill/is-type");

byteLength = require("./byteLength");

{
  browser = require("./browser");
}

noop = Promise.resolve();

htmlReg = /^\s*</;


/*
 * A default middleware that handles sending the response for the request.
 *
 * @param {Object} req
 * @param {Object} res
 * @param {Function} next
 */

module.exports = function(req, res, next) {
  var body, cookies, headers, method, originalRes, redirectURL, status, statusText;
  if (res.headersSent) {
    return noop;
  }
  method = req.method, redirectURL = req.redirectURL, cookies = req.cookies;
  originalRes = res.res, headers = res.headers, status = res.status, statusText = res.statusText, body = res.body;
  cookies.save();
  if (redirectURL != null) {
    if (headers["location"] == null) {
      headers["location"] = redirectURL;
    }
    if (!statuses.redirect[status]) {
      status = 302;
    }
    {
      browser.goTo(redirectURL);
      return noop;
    }
  }
  if (Number(status) === 404 && (body != null)) {
    status = 200;
  }
  if (statusText == null) {
    statusText = statuses[status];
  }
  switch (false) {
    case !(statuses.empty[status] || (body == null)):
      delete headers["content-type"];
      body = void 0;
      break;
    case !isType.String(body):
      if (headers["content-type"] == null) {
        headers["content-type"] = "text/" + (htmlReg.test(body) ? "html" : "plain") + "; charset=UTF-8";
      }
      break;
    case !isType.Buffer(body):
      headers["content-length"] = body.length;
      break;
    case !isType.Stream(body):
      if (headers["content-type"] == null) {
        headers["content-type"] = "application/octet-stream";
      }
      break;
    default:
      try {
        body = JSON.stringify(body);
        if (headers["content-type"] == null) {
          headers["content-type"] = "application/json; charset=UTF-8";
        }
      } catch (_error) {}
  }
  return noop;
};
