/** Rill v0.0.647 https://www.npmjs.com/package/rill License: MIT*/
"use strict";
var Cookies, Request, browserCookies, expect, parseURL;

Cookies = require("@rill/cookies");

parseURL = require("@rill/parse-url");

expect = require("./util/expect");


/*
 * Constructor that will normalize a request object.
 *
 * @param {Object} req
 * @constructor
 */

Request = (function() {
  function Request(req) {
    var defaults, key, ref, ref1, ref2, ref3, ref4, ref5, ref6, ref7, ref8, ref9, val;
    if (!(this instanceof Request)) {
      return new Request(req);
    }
    expect(req != null, "Could not create Request with an empty object.");
    expect(req.url != null, "Could not create a Request without a URL.");
    this.method = ((ref = req.method) != null ? ref : "GET").toUpperCase();
    this.headers = (ref1 = req.headers) != null ? ref1 : {};
    this.files = (ref2 = req.files) != null ? ref2 : [];
    this.body = (ref3 = req.body) != null ? ref3 : {};
    this.req = req;
    {
      defaults = this;
      this.ip = (ref4 = (ref5 = this.headers["x-forwarded-for"]) != null ? ref5.split(",")[0] : void 0) != null ? ref4 : (ref6 = req.connection) != null ? ref6.remoteAddress : void 0;
      ref7 = (this.headers["x-forwarded-host"] || this.headers["host"] || "").split(":"), this.hostname = ref7[0], this.port = ref7[1];
      this.protocol = ((ref8 = req.socket) != null ? ref8.encrypted : void 0) ? "https:" : "http:";
    }
    this.headers["referrer"] = this.headers["referer"];
    this.cookies = browserCookies || new Cookies(this);
    ref9 = parseURL(req.url, defaults);
    for (key in ref9) {
      val = ref9[key];
      this[key] = val;
    }
  }


  /*
  	 * Trigger another request that will redirect the browser and send a location change on the server.
  	 * If "back" is the url then it will attempt to redirect the user back a url, otherwise the alt will be used.
  	 *
  	 * @param {String} url
  	 * @param {String} alt
   */

  Request.prototype.redirect = function(url, alt) {
    if (url == null) {
      url = "back";
    }
    if (alt == null) {
      alt = "/";
    }
    return this.redirectURL = (url !== "back" ? url : this.headers.referer || alt);
  };

  return Request;

})();

module.exports = Request;
