serialize   = require("@rill/form-json")
parseURL    = require("@rill/parse-url")
expect      = require("../util/expect")
referrer    = document.referrer
hashReg     = /#.+$/
initialized = false

# Store registered createServer callback.
handler = ->
element = null

browser =
	###
	# Listen to all url change events on a dom element and trigger the server callback.
	#
	# @param {DOMNode} element
	# @api private
	###
	listen: (entity = global)->
		expect(not element?, "Can only run one server at a time.")
		element = entity
		global.addEventListener("DOMContentLoaded", handleURLChange)
		global.addEventListener("popstate", handleURLChange)
		element.addEventListener("submit", handleForm)
		element.addEventListener("click", handleLink)
		return

	###
	# Add a handler to listen to url changes.
	#
	# @param {Function} fn
	# @api private
	###
	createServer: (fn)->
		handler = fn
		return browser

	###
	# Trigger the registered handler to navigate to a given url.
	#
	# @param {String} url
	# @param {Object} options
	# @api private
	###
	goTo: (url, { method, headers, body, files, title, replaceState } = {})->
		title          ?= ""
		method         ?= "GET"
		headers        ?= {}
		headers.referer = referrer # Why am I subject to "referer" :(.
		referrer        = url

		# Create a mock http request to bootstrap the browser.
		handler({ url, body, files, method, headers }).then(({ redirected } = {})->
			# Update browser url unless the request was redirected.
			if not redirected and method is "GET"
				###
				# When navigating a user will be brought to the top of the page.
				# If the urls contains a hash that is the id of an element (a target) then the target will be scrolled to.
				# This is similar to how browsers handle page transitions natively.
				###

				if hash = url.match(hashReg)?[0]
					target = document.getElementById(hash[1..])
					target.scrollIntoView(block: "start", behavior: "smooth") if target
				else if initialized then global.scrollTo(0, 0)

				if replaceState then history.replaceState(null, title, url)
				else history.pushState(null, title, url)

				initialized = true
			return
		)

###
# Handle an event that changed the url (popstate or page load)
#
# @param {Object} event
###
handleURLChange = (event)->
	browser.goTo(location.href,
		method: "GET",
		replaceState: true
	)

###
# Handle intercepting link clicks to update the url.
#
# @param {Object} event
###
handleLink = (event)->
	# Ignore canceled events, modified clicks, and right clicks.
	return if event.defaultPrevented or
			  event.metaKey  or
			  event.ctrlKey  or
			  event.shiftKey or
			  event.button   isnt 0

	# Get the <a> element.
	el = event.target
	el = el.parentNode while el? and el.nodeName isnt "A"

	# Ignore clicks from non-a elements.
	return unless el?

	# Ignore the click if the element has a target.
	return if el.target and el.target isnt "_self"

	# Ignore the click if it's a download link. (We use this method of
	# detecting the presence of the attribute for old IE versions.)
	return if Boolean(el.attributes.download)

	# Use a url parser to parse URLs instead of relying on the browser
	# to do it for us (because IE).
	url = el.href
	{ protocol, host } = parseURL(url, el)

	# Ignore links that don't share a protocol and host with ours.
	return if location.protocol.indexOf(protocol) is -1 or location.host isnt host

	# Ignore 'rel="external"' links.
	return if el.rel and /(?:^|\s+)external(?:\s+|$)/.test(el.rel)

	browser.goTo(url)
	event.preventDefault()

###
# Handle intercepting forms to update the url.
#
# @param {Object} event
###
handleForm = (event)->
	# Ignore canceled events, modified clicks, and right clicks.
	return if event.defaultPrevented or
			  event.metaKey or
			  event.ctrlKey or
			  event.shiftKey

	# Get the <form> element.
	el = event.target
	el = el.parentNode while el? and el.nodeName isnt "FORM"

	# Ignore clicks from non-form elements.
	return unless el?

	# Ignore the click if the element has a target.
	return if el.target and el.target isnt "_self"

	# Use a url parser to parse URLs instead of relying on the browser
	# to do it for us (because IE).
	url = el.action
	{ protocol, host } = parseURL(url, location)

	# Ignore links that don't share a protocol and host with ours.
	return if location.protocol.indexOf(protocol) is -1 or location.host isnt host

	# Ignore 'rel="external"' links.
	return if el.rel and /(?:^|\s+)external(?:\s+|$)/.test(el.rel)

	{ body, files } = serialize(el)
	method          = (el.getAttribute("method") or el.method).toUpperCase()
	el.reset() unless el.hasAttribute("data-noreset")

	if method is "GET"
		# Ignore qs when using GET (consistent with browsers).
		url   = "#{url.split("?")[0]}?#{JSON.stringify(body)}"
		body  = {}
		files = []

	browser.goTo(url, { method, body, files })
	event.preventDefault()

module.exports = browser
