statuses   = require("statuses")
isType     = require("@rill/is-type")
byteLength = require("./byteLength")
browser    = require("./browser") if process.browser
noop       = Promise.resolve()
htmlReg    = /^\s*</

###
# A default middleware that handles sending the response for the request.
#
# @param {Object} req
# @param {Object} res
# @param {Function} next
###
module.exports = (req, res, next)->
	# Skip request if something had already ended it.
	return noop if res.headersSent

	{ method, redirectURL, cookies }                        = req
	{ res: originalRes, headers, status, statusText, body } = res

	# Persist cookies.
	cookies.save()

	# Redirects isomorphically.
	if redirectURL?
		headers["location"] ?= redirectURL
		status = 302 unless statuses.redirect[status]

		if process.browser
			browser.goTo(redirectURL)
			return noop

	# Default the status to 200 if there is substance to the response.
	status      = 200 if Number(status) is 404 and body?
	statusText ?= statuses[status]

	# Ensure appropriate content-type and length.
	switch
		when statuses.empty[status] or not body?
			delete headers["content-type"]
			body = undefined
		when isType.String(body)
			headers["content-type"] ?= "text/#{if htmlReg.test(body) then "html" else "plain"}; charset=UTF-8"
		when isType.Buffer(body)
			headers["content-length"] = body.length
		when isType.Stream(body)
			headers["content-type"] ?= "application/octet-stream"
		else try
			body                    = JSON.stringify(body)
			headers["content-type"] ?= "application/json; charset=UTF-8"

	unless process.browser
		# Send response in server environment.
		originalRes.writeHead(status, statusText, headers)
		switch
			when "HEAD" is method then originalRes.end()
			when isType.Stream(body) then body.pipe(originalRes)
			else
				headers["content-length"] ?= byteLength(body)
				originalRes.end(body)
	return noop

